package sun.java2d.pipe;

import sun.java2d.loops.XORComposite;
import java.awt.AlphaComposite;
import sun.java2d.InvalidPipeException;
import java.awt.Color;
import sun.java2d.SunGraphics2D;
import java.lang.ref.WeakReference;
import java.awt.geom.AffineTransform;
import java.awt.Paint;
import java.awt.Composite;
import sun.java2d.pipe.hw.AccelSurface;
import java.lang.ref.Reference;

public abstract class BufferedContext
{
    public static final int NO_CONTEXT_FLAGS = 0;
    public static final int SRC_IS_OPAQUE = 1;
    public static final int USE_MASK = 2;
    protected RenderQueue rq;
    protected RenderBuffer buf;
    protected static BufferedContext currentContext;
    private Reference<AccelSurface> validSrcDataRef;
    private Reference<AccelSurface> validDstDataRef;
    private Reference<Region> validClipRef;
    private Reference<Composite> validCompRef;
    private Reference<Paint> validPaintRef;
    private boolean isValidatedPaintJustAColor;
    private int validatedRGB;
    private int validatedFlags;
    private boolean xformInUse;
    private AffineTransform transform;
    
    protected BufferedContext(RenderQueue rq) {
        this.validSrcDataRef = new WeakReference(null);
        this.validDstDataRef = new WeakReference(null);
        this.validClipRef = new WeakReference(null);
        this.validCompRef = new WeakReference(null);
        this.validPaintRef = new WeakReference(null);
        this.rq = rq;
        this.buf = rq.getBuffer();
    }
    
    public static void validateContext(AccelSurface accelSurface, AccelSurface accelSurface2, Region region, Composite composite, AffineTransform affineTransform, Paint paint, SunGraphics2D sunGraphics2D, int n) {
        accelSurface2.getContext().validate(accelSurface, accelSurface2, region, composite, affineTransform, paint, sunGraphics2D, n);
    }
    
    public static void validateContext(AccelSurface accelSurface) {
        validateContext(accelSurface, accelSurface, null, null, null, null, null, 0);
    }
    
    public void validate(AccelSurface referent, AccelSurface referent2, Region region, Composite referent3, AffineTransform transform, Paint referent4, SunGraphics2D sunGraphics2D, int validatedFlags) {
        boolean n = false;
        boolean n2 = false;
        if (!referent2.isValid() || referent2.isSurfaceLost() || referent.isSurfaceLost()) {
            this.invalidateContext();
            throw new InvalidPipeException("bounds changed or surface lost");
        }
        if ((referent4 instanceof Color)) {
            int rgb = ((Color)referent4).getRGB();
            if (this.isValidatedPaintJustAColor) {
                if (rgb != this.validatedRGB) {
                    this.validatedRGB = rgb;
                    n2 = true;
                }
            }
            else {
                this.validatedRGB = rgb;
                n2 = true;
                this.isValidatedPaintJustAColor = true;
            }
        }
        else if (this.validPaintRef.get() != referent4) {
            n2 = true;
            this.isValidatedPaintJustAColor = false;
        }
        AccelSurface accelSurface = (AccelSurface)this.validSrcDataRef.get();
        AccelSurface accelSurface2 = (AccelSurface)this.validDstDataRef.get();
        if (BufferedContext.currentContext != this || referent != accelSurface || referent2 != accelSurface2) {
            if (referent2 != accelSurface2) {
                n = true;
            }
            if (referent4 == null) {
                n2 = true;
            }
            this.setSurfaces(referent, referent2);
            BufferedContext.currentContext = this;
            this.validSrcDataRef = new WeakReference(referent);
            this.validDstDataRef = new WeakReference(referent2);
        }
        Region region2 = (Region)this.validClipRef.get();
        if (region != region2 || n) {
            if (region != null) {
                if (n || region2 == null || !region2.isRectangular() || !region.isRectangular() || region.getLoX() != region2.getLoX() || region.getLoY() != region2.getLoY() || region.getHiX() != region2.getHiX() || region.getHiY() != region2.getHiY()) {
                    this.setClip(region);
                }
            }
            else {
                this.resetClip();
            }
            this.validClipRef = new WeakReference(region);
        }
        if (referent3 != this.validCompRef.get() || validatedFlags != this.validatedFlags) {
            if (referent3 != null) {
                this.setComposite(referent3, validatedFlags);
            }
            else {
                this.resetComposite();
            }
            n2 = true;
            this.validCompRef = new WeakReference(referent3);
            this.validatedFlags = validatedFlags;
        }
        boolean n3 = false;
        if (transform == null) {
            if (this.xformInUse) {
                this.resetTransform();
                this.xformInUse = false;
                n3 = true;
            }
            else if (sunGraphics2D != null && !sunGraphics2D.transform.equals(this.transform)) {
                n3 = true;
            }
            if (sunGraphics2D != null && n3) {
                this.transform = new AffineTransform(sunGraphics2D.transform);
            }
        }
        else {
            this.setTransform(transform);
            this.xformInUse = true;
            n3 = true;
        }
        if (!this.isValidatedPaintJustAColor && n3) {
            n2 = true;
        }
        if (!(n2 == 0)) {
            if (referent4 != null) {
                BufferedPaints.setPaint(this.rq, sunGraphics2D, referent4, validatedFlags);
            }
            else {
                BufferedPaints.resetPaint(this.rq);
            }
            this.validPaintRef = new WeakReference(referent4);
        }
        referent2.markDirty();
        return;
    }
    
    private void invalidateSurfaces() {
        this.validSrcDataRef.clear();
        this.validDstDataRef.clear();
    }
    
    private void setSurfaces(AccelSurface accelSurface, AccelSurface accelSurface2) {
        this.rq.ensureCapacityAndAlignment(20, 4);
        this.buf.putInt(70);
        this.buf.putLong(accelSurface.getNativeOps());
        this.buf.putLong(accelSurface2.getNativeOps());
    }
    
    private void resetClip() {
        this.rq.ensureCapacity(4);
        this.buf.putInt(55);
    }
    
    private void setClip(Region region) {
        if (region.isRectangular()) {
            this.rq.ensureCapacity(20);
            this.buf.putInt(51);
            this.buf.putInt(region.getLoX()).putInt(region.getLoY());
            this.buf.putInt(region.getHiX()).putInt(region.getHiY());
        }
        else {
            this.rq.ensureCapacity(28);
            this.buf.putInt(52);
            this.buf.putInt(53);
            int n = this.buf.position();
            this.buf.putInt(0);
            int n2 = 0;
            int n3 = this.buf.remaining() / 16;
            for (int[] array = new int[4]; region.getSpanIterator().nextSpan(array); ++n2, --n3) {
                if (n3 == 0) {
                    this.buf.putInt(n, n2);
                    this.rq.flushNow();
                    this.buf.putInt(53);
                    n = this.buf.position();
                    this.buf.putInt(0);
                    n2 = 0;
                    n3 = this.buf.remaining() / 16;
                }
                this.buf.putInt(array[0]);
                this.buf.putInt(array[1]);
                this.buf.putInt(array[2]);
                this.buf.putInt(array[3]);
            }
            this.buf.putInt(n, n2);
            this.rq.ensureCapacity(4);
            this.buf.putInt(54);
        }
    }
    
    private void resetComposite() {
        this.rq.ensureCapacity(4);
        this.buf.putInt(58);
    }
    
    private void setComposite(Composite composite, int n) {
        if ((composite instanceof AlphaComposite)) {
            AlphaComposite alphaComposite = (AlphaComposite)composite;
            this.rq.ensureCapacity(16);
            this.buf.putInt(56);
            this.buf.putInt(alphaComposite.getRule());
            this.buf.putFloat(alphaComposite.getAlpha());
            this.buf.putInt(n);
        }
        else if ((composite instanceof XORComposite)) {
            int xorPixel = ((XORComposite)composite).getXorPixel();
            this.rq.ensureCapacity(8);
            this.buf.putInt(57);
            this.buf.putInt(xorPixel);
        }
        else {
            throw new InternalError("not yet implemented");
        }
    }
    
    private void resetTransform() {
        this.rq.ensureCapacity(4);
        this.buf.putInt(60);
    }
    
    private void setTransform(AffineTransform affineTransform) {
        this.rq.ensureCapacityAndAlignment(52, 4);
        this.buf.putInt(59);
        this.buf.putDouble(affineTransform.getScaleX());
        this.buf.putDouble(affineTransform.getShearY());
        this.buf.putDouble(affineTransform.getShearX());
        this.buf.putDouble(affineTransform.getScaleY());
        this.buf.putDouble(affineTransform.getTranslateX());
        this.buf.putDouble(affineTransform.getTranslateY());
    }
    
    public void invalidateContext() {
        this.resetTransform();
        this.resetComposite();
        this.resetClip();
        BufferedPaints.resetPaint(this.rq);
        this.invalidateSurfaces();
        this.validCompRef.clear();
        this.validClipRef.clear();
        this.validPaintRef.clear();
        this.isValidatedPaintJustAColor = false;
        this.xformInUse = false;
    }
    
    public abstract RenderQueue getRenderQueue();
    
    public abstract void saveState();
    
    public abstract void restoreState();
}
